@@ -1,4 +1,5 @@
 use core::{
+    borrow::Borrow,
     fmt,
     fmt::{Binary, Debug, Display, Formatter, LowerExp, LowerHex, Octal, UpperExp, UpperHex},
     num::NonZeroUsize,
@@ -7,8 +8,8 @@ use core::{
 
 use crate::{FromIndex, Idx, IntoIndex, NewIndex};
 
-/// This is partly like [`Cow`], but the 'owned' type is the same as the 'borrowed'
-/// type.
+/// This is partly like [`Cow`], but the 'owned' type is always the same as the
+/// 'borrowed' type.
 ///
 /// [`Cow`]: alloc::borrow::Cow
 #[cfg_attr(feature = "gc", derive(gc::Trace))]
@@ -18,29 +19,34 @@ pub enum Key<'a, T> {
     Borrowed(&'a T),
 }
 
-impl<'a, T> Deref for Key<'a, T> {
+impl<T> Deref for Key<'_, T> {
     type Target = T;
     fn deref(&self) -> &T {
         match *self {
-            Key::Owned(ref t) => t,
-            Key::Borrowed(t) => t,
+            Key::Owned(ref t) | Key::Borrowed(&ref t) => t,
         }
     }
 }
 
-impl<'a, T: IntoIndex> IntoIndex for Key<'a, T> {
-    fn into_index(&self) -> Option<Idx<Self>> {
-        Some(unsafe { (**self).into_index()?.cast() })
+impl<T> Borrow<T> for Key<'_, T> {
+    fn borrow(&self) -> &T {
+        self
     }
 }
 
-impl<'a, T: FromIndex> FromIndex for Key<'a, T> {
+impl<T: IntoIndex> IntoIndex for Key<'_, T> {
+    fn into_index(&self) -> Option<Idx<Self>> {
+        Some(T::into_index(self)?.cast_safe())
+    }
+}
+
+impl<T: FromIndex> FromIndex for Key<'_, T> {
     fn from_index(v: Idx<Self>) -> Self {
         Self::Owned(T::from_index(unsafe { v.cast() }))
     }
 }
 
-unsafe impl<'a, T: NewIndex> NewIndex for Key<'a, T> {
+unsafe impl<T: NewIndex> NewIndex for Key<'_, T> {
     fn new_index_allowed(idx: NonZeroUsize) -> bool {
         T::new_index_allowed(idx)
     }
