@@ -131,7 +131,7 @@ type Nums = [usize; NUMS_LEN];
 /// It is a logic error for a key to be modified in such a way that the key's
 /// hash, as determined by the [`Hash`] trait, or its equality, as determined by
 /// the [`Eq`] trait, changes while it is in the map. This is normally only
-/// possible through [`Cell`], [`RefCell`], global state, I/O, or unsafe code.
+/// possible through [`Cell`], [`RefCell`], global state, or I/O.
 ///
 /// # Examples
 ///
@@ -538,7 +538,7 @@ impl<K: FromIndex + Hash + Eq, V> Ham<K, V> {
     {
         if let Some((idx, v)) = self.get_array_idx(key) {
             v.as_ref()
-                .map(|v| (Key::from_index(unsafe { idx.cast() }), v))
+                .map(|v| (Key::Owned(idx.cast_safe().into_value()), v))
         } else {
             self.get_hash(key)
                 .and_then(|v| v.value().as_ref().map(|(k, v)| (Key::Borrowed(k), v)))
@@ -568,7 +568,7 @@ impl<K: FromIndex + Hash + Eq, V> Ham<K, V> {
         Q: IntoIndex + Hash + Eq,
     {
         let v = if let Some((idx, v)) = self.get_array_idx_mut(key) {
-            v.take().map(|v| (K::from_index(unsafe { idx.cast() }), v))
+            v.take().map(|v| (idx.cast_safe().into_value(), v))
         } else {
             self.get_hash_take(key)
         };
@@ -626,9 +626,8 @@ impl<K: NewIndex + Hash + Eq, V> Ham<K, V> {
             {
                 if max == usize::MAX {
                     return Some(usize::MAX);
-                } else {
-                    max = max.saturating_mul(2);
                 }
+                max = max.saturating_mul(2);
             }
 
             binary_search(min, max, |i| {
@@ -731,7 +730,7 @@ impl<K: IntoIndex + FromIndex + Hash + Eq, V> Ham<K, V> {
     fn insert_and_get_mut(&mut self, key: K, value: V) -> &mut V {
         if let Some(idx) = self.get_array_idx_only(&key) {
             // To get it past the borrow checker...
-            let slot = unsafe { self.array.get_unchecked_mut(idx) };
+            let slot = &mut self.array[idx];
 
             if slot.is_none() {
                 self.len += 1;
@@ -740,7 +739,7 @@ impl<K: IntoIndex + FromIndex + Hash + Eq, V> Ham<K, V> {
             slot.insert(value)
         } else if let Some(idx) = self.get_hash_idx_only(&key) {
             // To get it past the borrow checker...
-            let node = unsafe { self.hash.get_unchecked_mut(idx) };
+            let node = &mut self.hash[idx];
 
             let entry = node.value_mut().take();
 
@@ -870,12 +869,9 @@ impl<K: IntoIndex + FromIndex + Hash + Eq, V> Ham<K, V> {
     // @@  @@  @@      @@  @@  @@  @@      @@  @@  @@
     // @@  @@  @@@@@@  @@  @@  @@  @@  @@@@@@  @@  @@
     fn count_int(key: &K, nums: &mut Nums) -> bool {
-        if let Some(v) = key.into_index() {
-            nums[ceil_log_2(v.get())] += 1;
-            true
-        } else {
-            false
-        }
+        key.into_index()
+            .map(|v| nums[ceil_log_2(v.get())] += 1)
+            .is_some()
     }
 
     // The number of items that can be in an array that are in the hash part.
@@ -1123,9 +1119,8 @@ impl<K, V> Ham<K, V> {
                 let new_v = Node::next(v);
                 if new_v as *const _ == v {
                     return None; // There wasn't a chain.
-                } else {
-                    v = new_v;
                 }
+                v = new_v;
             }
             Some(unsafe { &mut *v.as_ptr() })
         }
@@ -1146,10 +1141,9 @@ impl<K, V> Ham<K, V> {
                 let new_v = Node::next(v);
                 if new_v as *const _ == v {
                     return None; // There wasn't a chain.
-                } else {
-                    prev_v = Some(v);
-                    v = new_v;
                 }
+                prev_v = Some(v);
+                v = new_v;
             }
 
             Some(if let Some(prev) = prev_v {
@@ -1177,15 +1171,15 @@ impl<K, V> Ham<K, V> {
                 if new_v as *const _ == v {
                     // v -> v -> v ...
                     return None; // There wasn't a chain.
-                } else {
-                    prev_v = Some(v);
-                    v = new_v;
                 }
+                prev_v = Some(v);
+                v = new_v;
             }
 
             let v_next = Node::next(v);
 
             {
+                #[allow(clippy::cast_sign_loss)]
                 let num = unsafe { v.as_ptr().offset_from(self.hash.as_ptr()) } as usize;
                 if num > self.last_free {
                     self.last_free = num;
@@ -1193,14 +1187,14 @@ impl<K, V> Ham<K, V> {
             }
 
             let out = if let Some(prev_v) = prev_v {
-                if v_next as *const _ != v {
-                    // prev_v -> v -> v_next -> ...
-                    Node::set_next(prev_v, v_next);
-                    // prev_v -> v_next -> ...; v -> v_next -> ...
-                } else {
+                if v_next as *const _ == v {
                     // prev_v -> v
                     Node::set_next(prev_v, prev_v);
                     // prev_v; v
+                } else {
+                    // prev_v -> v -> v_next -> ...
+                    Node::set_next(prev_v, v_next);
+                    // prev_v -> v_next -> ...; v -> v_next -> ...
                 }
 
                 unsafe { &mut *v.as_ptr() }.value_mut().take()
@@ -1252,9 +1246,8 @@ impl<K, V> Ham<K, V> {
                 let new_v = v.next_ref();
                 if new_v as *const _ == v {
                     return None; // There wasn't a chain.
-                } else {
-                    v = new_v;
                 }
+                v = new_v;
             }
             Some(v)
         }
@@ -1281,10 +1274,10 @@ impl<K, V> Ham<K, V> {
                 let new_v = v.next_ref();
                 if new_v as *const _ == v {
                     return None; // There wasn't a chain.
-                } else {
-                    v = new_v;
                 }
+                v = new_v;
             }
+            #[allow(clippy::cast_sign_loss)]
             Some((
                 unsafe { (v as *const Node<K, V>).offset_from(self.hash.as_ptr()) } as usize,
                 v,
@@ -1306,8 +1299,7 @@ impl<K, V> Ham<K, V> {
 
         let idx = (hasher.finish() as usize) % self.hash.len();
 
-        // SAFETY: idx < self.hash.len()
-        unsafe { self.hash.get_unchecked(idx) }
+        &self.hash[idx]
     }
 
     /// Returns a reference to the position of a key in the table
@@ -1392,7 +1384,7 @@ where
     where
         I: IntoIterator<Item = (K, V)>,
     {
-        for (k, v) in iter.into_iter() {
+        for (k, v) in iter {
             self.insert(k, v);
         }
     }
