@@ -27,12 +27,11 @@ use core::{
 ///
 /// # Valid indices
 ///
-/// Valid indices are defined by the type `T`. In order for an index
-/// to be valid, they must be valid to be passed into
-/// [`FromIndex::from_index`].
+/// Valid indices are defined by the type `T` and its borrowed forms.
+/// In order for an index to be valid, it must be allowed to be passed
+/// into [`FromIndex::from_index`] without causing UB.
 ///
 /// Created by either [`IntoIndex::into_index`] or [`Idx::try_new`].
-#[must_use]
 #[derive(Eq, PartialEq, Debug)]
 pub struct Idx<T: ?Sized> {
     v: NonZeroUsize,
@@ -57,6 +56,7 @@ impl<T: ?Sized> fmt::Display for Idx<T> {
 impl<T: ?Sized + NewIndex> Idx<T> {
     /// Create a new index with the specified value if the type allows
     /// it.
+    #[must_use]
     pub fn try_new(v: NonZeroUsize) -> Option<Self> {
         if <T as NewIndex>::new_index_allowed(v) {
             // SAFETY: New indices with that value are allowed.
@@ -68,6 +68,7 @@ impl<T: ?Sized + NewIndex> Idx<T> {
 
     /// Create a new index with the specified value if the type allows
     /// it.
+    #[must_use]
     crate fn try_new_usize(v: usize) -> Option<Self> {
         NonZeroUsize::new(v).and_then(Self::try_new)
     }
@@ -81,6 +82,7 @@ impl<T: ?Sized> Idx<T> {
     /// The value must be a [valid index][idx].
     ///
     /// [idx]: #valid-indices
+    #[must_use]
     pub const unsafe fn new(v: NonZeroUsize) -> Self {
         Self {
             v,
@@ -96,17 +98,16 @@ impl<T: ?Sized> Idx<T> {
     /// The value must either be a [valid index][idx] or zero.
     ///
     /// [idx]: #valid-indices
+    #[must_use]
     pub const unsafe fn from_usize(v: usize) -> Option<Self> {
         if let Some(v) = NonZeroUsize::new(v) {
-            Some(Self {
-                v,
-                _phantom: PhantomData,
-            })
+            Some(unsafe { Self::new(v)            })
         } else {
             None
         }
     }
 
+    #[must_use]
     pub const fn get(&self) -> NonZeroUsize {
         self.v
     }
@@ -119,15 +120,14 @@ impl<T: ?Sized> Idx<T> {
     /// The index must be a [valid index][idx] for type `U`.
     ///
     /// [idx]: #valid-indices
+    #[must_use]
     pub const unsafe fn cast<U: ?Sized>(self) -> Idx<U> {
         unsafe { Idx::new(self.v) }
     }
 
+    #[must_use]
     crate const unsafe fn new_unchecked(v: usize) -> Self {
-        Self {
-            v: unsafe { NonZeroUsize::new_unchecked(v) },
-            _phantom: PhantomData,
-        }
+        unsafe { Self::new(NonZeroUsize::new_unchecked(v)) }
     }
 }
 
@@ -135,7 +135,8 @@ impl<T: ?Sized> Idx<T> {
     /// This reinterprets the index as a different type of index;
     /// essentially `Idx::<U>::new(self.get())`.
     ///
-    /// This is restricted by Borrow, meaning that it is type-safe.
+    /// This is restricted by [`Borrow`], meaning that it is type-safe.
+    #[must_use]
     pub fn cast_safe<U: Borrow<T>>(self) -> Idx<U> {
         unsafe { Idx::new(self.v) }
     }
@@ -147,6 +148,7 @@ impl<T: ?Sized + FromIndex> Idx<T> {
     /// A shorthand for [`<T as FromIndex>::from_index(self)`][fi].
     ///
     /// [fi]: FromIndex::from_index
+    #[must_use]
     pub fn into_value(self) -> T {
         <T as FromIndex>::from_index(self)
     }
@@ -156,6 +158,7 @@ impl<T: ?Sized + FromIndex> Idx<T> {
 pub unsafe trait NewIndex: FromIndex + IntoIndex {
     /// Whether the specified value is allowed to be used to make an
     /// index out of thin air.
+    #[must_use]
     fn new_index_allowed(v: NonZeroUsize) -> bool;
 }
 
@@ -186,6 +189,14 @@ pub unsafe trait NewIndex: FromIndex + IntoIndex {
 ///         Self(idx.get().get() as i32)
 ///     }
 /// }
+///
+/// // Allow creating new indices out of thin air.
+/// // To let `Ham::get_n` work.
+/// unsafe impl NewIndex for CouldBeAnIdx {
+///     fn new_index_allowed(idx: NonZeroUsize) -> bool {
+///         i32::try_from(idx.get()).is_ok()
+///     }
+/// }
 /// ```
 pub trait IntoIndex {
     /// Try to convert `self` into an [`Idx`].
@@ -203,24 +214,25 @@ pub trait IntoIndex {
     /// );
     /// ```
     #[allow(clippy::wrong_self_convention)]
+    #[must_use]
     fn into_index(&self) -> Option<Idx<Self>>;
 }
 
-/// Conversion from an index.
+/// Conversion from an [`Idx`].
 ///
-/// This trait implements conversion from an index into `Self`.
+/// This trait implements conversion from an `Idx` into `Self`.
 pub trait FromIndex: Sized + IntoIndex {
     /// Convert it.
     ///
     /// # Implementors
     ///
     /// The only guarantees about the index value is that it has either
-    /// come from `Self::into_index`, `Idx::try_new` or that is has
-    /// come from a borrowed form of `Self`.
+    /// come from [`IntoIndex::into_index`], [`Idx::try_new`] or that it
+    /// has come from a borrowed form of `Self`.
     ///
     /// By implementing `Borrow<T>` for your type, this loosens the
-    /// guarantees given here, since this index could also come from
-    /// `T`.
+    /// guarantees given here, since this index could also come from `T`.
+    #[must_use]
     fn from_index(idx: Idx<Self>) -> Self;
 }
 
@@ -241,7 +253,7 @@ macro_rules! impl_test_index {
 
             unsafe impl NewIndex for $t {
                 fn new_index_allowed(idx: NonZeroUsize) -> bool {
-                    idx.get() <= (<$t>::MAX as usize)
+                    <$t>::try_from(idx.get()).is_ok()
                 }
             }
         )+
@@ -332,13 +344,13 @@ unsafe impl NewIndex for char {
 // Note: There is no matching FromIndex.
 impl<T: IntoIndex> IntoIndex for &T {
     fn into_index(&self) -> Option<Idx<Self>> {
-        Some((**self).into_index()?.cast_safe())
+        Some(T::into_index(self)?.cast_safe())
     }
 }
 
 impl<T: IntoIndex> IntoIndex for Box<T> {
     fn into_index(&self) -> Option<Idx<Self>> {
-        Some((**self).into_index()?.cast_safe())
+        Some(T::into_index(self)?.cast_safe())
     }
 }
 
@@ -392,7 +404,7 @@ unsafe impl<T: NewIndex, E> NewIndex for Result<T, E> {
 
 impl<T: ?Sized + IntoIndex + ToOwned> IntoIndex for Cow<'_, T> {
     fn into_index(&self) -> Option<Idx<Self>> {
-        Some((**self).into_index()?.cast_safe())
+        Some(T::into_index(self)?.cast_safe())
     }
 }
 
